/********************************************
 * Istruzioni per la compilazione
 * arm-linux-gnueabihf-gcc main.c -o main 	\
 *   -I/opt/altera/15.0/embedded/ip/altera/hps/altera_hps/hwlib/include/ 	\
 *   -I/opt/altera/15.0/embedded/ip/altera/hps/altera_hps/hwlib/include/soc_cv_av/
 * ******************************************/

#ifndef soc_cv_av
#define	soc_cv_av
#endif

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <time.h>
#include <sys/mman.h>
#include "hwlib.h"
#include "socal/socal.h"
#include "socal/hps.h"
#include "socal/alt_gpio.h"
#include "hps_0.h"

#define HW_REGS_SPAN ( 0x04000000 )
#define HW_REGS_MASK ( HW_REGS_SPAN - 1 )

volatile unsigned long *h2p_lw_led_addr=NULL;


int main(int argc, char **argv)
{
	void *virtual_base;
	int fd;
	unsigned led;

	printf("ALT_STM_OFST = 0x%8X\nALT_LWFPGASLVS_OFST = 0x%8X\nLED_BASE = 0x%0X\n(ALT_LWFPGASLVS_OFST + LED_BASE) & HW_REGS_MASK = 0x%08X\n\n",
		   ALT_STM_OFST, ALT_LWFPGASLVS_OFST, LED_BASE, (ALT_LWFPGASLVS_OFST + LED_BASE) & (HW_REGS_MASK));

	// Apertura del device che permette l'accesso diretto alla memoria
	if((fd = open("/dev/mem", (O_RDWR | O_SYNC))) == -1) {
		printf("ERROR: could not open \"/dev/mem\"...\n");
		return(1);
	}
	
	// Mappatura della memria a partire da ALT_STM_OFST (System Trace Macrocell (STM) Module Address Map)
	// Questo indirizzo è esattamente qeullo di inizio dell'area destinata alle periferiche: 0xFC00_0000
	virtual_base = mmap( NULL, HW_REGS_SPAN, (PROT_READ | PROT_WRITE), MAP_SHARED, fd, ALT_STM_OFST);	
	if(virtual_base == MAP_FAILED) {
		printf("ERROR: mmap() failed...\n");
		close(fd);
		return(1);
	}
	
	h2p_lw_led_addr = virtual_base + (ALT_LWFPGASLVS_OFST-ALT_STM_OFST+LED_BASE);

	if(argc > 1)
	{
		led = atoi(argv[1]);
		printf("%s\n", argv[1]);
	}
	else
	{
		printf("Azzeramento\n");
		led = 0;
	}

	*h2p_lw_led_addr = led;

	if(munmap(virtual_base, HW_REGS_SPAN ) != 0) {
		printf("ERROR: munmap() failed...\n");
		close(fd);
		return 1;
	}

	close(fd);
	return 0;
}